using System;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;

namespace dict {

  /// <summary>Common purpose utility functions</summary>
  public class Utils {

#region PInvoke
    /// <summary>Resets idle time and prevents suspending</summary>
    /// <remarks>Only on windows ce/mobile</remarks>
    [DllImport("coredll.dll",
               EntryPoint="SystemIdleTimerReset",
               SetLastError=false)]
    private static extern void SystemIdleTimerReset();

    /// <summary>Switches SIP or on off</summary>
    /// <remarks>Only on Windows CE/mobile</remarks>
    /// <param name="status">0 for off, 1 for on</param>
    /// <returns>true for success</returns>
    [DllImport("coredll.dll",
               EntryPoint="SipShowIM",
               SetLastError=false)]
    private static extern bool SipShowIM(int status);
#endregion

    /// <summary>Checks if running under MS Windows</summary>
    /// <returns>true for windows</returns>
    public static bool isWindows() {
      bool retval;

      switch(Environment.OSVersion.Platform) {
        case PlatformID.Win32NT :
        case PlatformID.Win32S :
        case PlatformID.Win32Windows :
        case PlatformID.WinCE :
          retval = true;
          break;
        default :
          retval = false;
          break;
      } // switch
      return retval;
    } // isWindows()

    /// <summary>Checks if the platform is windows ce</summary>
    /// <returns>True for windows ce</returns>
    public static bool isMobile() {
      if (Environment.OSVersion.Platform == PlatformID.WinCE) return true;
      return false;
    } // isMobile()

    /// <summary>Causes that the system won't suspend</summary>
    /// <remarks>
    /// Only on windows ce/mobile. For other platforms the method does nothing.
    /// </remarks>
    public static void dontSuspend() {
      if (Utils.isMobile()) {
        try {
          Utils.SystemIdleTimerReset();
        }
        catch(Exception) {}
      }
    } // dontSuspend()

    /// <summary>Switches SIP on or off</summary>
    /// <remarks>
    /// Only on mobile platforms, else the method does nothing.
    /// On mobile platforms it uses SipShowIM() in order to show or hide
    /// the Soft Input Panel (SIP).
    /// </remarks>
    /// <param name="on">true for showing SIP, false for hiding it</param>
    /// <seealso cref="SipShowIM">SipShowIM</seealso>
    /// <seealso cref="isMobile">isMobile</seealso>
    public static void sip(bool on) {
      if (Utils.isMobile()) {
        if (on) Utils.SipShowIM(1); else Utils.SipShowIM(0);
      }
    } // sip()

    /// <summary>The new line character for the present platform</summary>
    public static string newline {
      get {
        if (Utils.isWindows()) return "\r\n";
        return "\n";
      }
    } // newline

    /// <summary>The full path to the executing assembly</summary>
    public static string Exepath {
      get {
        Uri    u;
        string s;

        s = Assembly.GetExecutingAssembly().GetName().CodeBase;
        u = new Uri(s);
        if (u.IsFile) s = u.LocalPath;
        return s;
      }
    } // Exepath

    /// <summary>The directory where the executing assembly is located</summary>
    public static string Exedir {
      get {
        return Path.GetDirectoryName(Utils.Exepath);
      }
    } // Exedir
  } // class Utils
} // namespace dict

